#include <iostream>
#include <vector>
#include <cstring>
#include <cstdio>
using namespace std;

typedef long long ll;
#define TASK "parades"

template <class HashType, int P, int ALPHABET, int LOG_N>
struct Trie {
#define SZ ((1<<LOG_N) + 5)

    class Node {
        friend class Trie;

        Node* jumps[LOG_N];
        Trie * trie;

    public:
        int lev, leafs;
        HashType hash;
        vector<const Node*> relaxes;

        Node* to[ALPHABET];
        Node() {}

        Node(Trie * trie): trie(trie), lev(0), leafs(0), hash(0) {
            memset(to, 0, sizeof to);
            memset(jumps, 0, sizeof jumps);
        }

        Node(Trie * trie, Node* parent, int c): lev(0), leafs(0), trie(trie) {
            memset(to, 0, sizeof to);
            memset(jumps, 0, sizeof jumps);
            hash = parent->hash * P + (c + 1);
            lev = parent->lev + 1;
            jumps[0] = parent;
            for (int h = 1; (1<<h) <= lev; ++h)
                jumps[h] = jumps[h - 1]->jumps[h - 1];
        }

        Node* calcSuffix(const vector<HashType>& hashT) {
            int l = 0;
            Node* v = this;
            Node* startV = v;
            HashType startHash = v->hash;

            for (int i = LOG_N - 1; i >= 0; --i)
                if (l + (1<<i) <= hashT.size() && v->jumps[i] != NULL) {
                    HashType hash = startV->hash - v->jumps[i]->hash * trie->pw[startV->lev - v->jumps[i]->lev];
                    if (hash == hashT[l + (1<<i) - 1]) {
                        l += 1 << i;
                        v = v->jumps[i];
                    }
                }
            return v;
        }

        void addVertex(const Node* v) {
            relaxes.push_back(v);
        }
    };

    HashType pw[SZ];

    Node *root;
    Trie(): ptr(0) {
        root = newNode();
        pw[0] = 1;
        for (int i = 1; i < SZ; ++i)
            pw[i] = pw[i - 1] * P;
    }

    Node* insert(const string& s) {
        Node *v = root;
        int len = s.size();
        for (int i = 0; i < len; ++i) {
            int c = s[i] - 'a';
            if (v->to[c] == NULL)
                v->to[c] = newNode(v, c);
            v = v->to[c];
            v->leafs++;
        }
        return v;
    }
private:

    Node* newNode(Node* v, int c) {
        mem[ptr++] = Node(this, v, c);
        return &mem[ptr - 1];
    }

    Node* newNode() {
        mem[ptr++] = Node(this);
        return &mem[ptr - 1];
    }

    Node mem[SZ];
    int ptr;
};

const int MAX_N = 1e5 + 10;
const int P = 31;
const int ALPHABET = 26;
const int LOG_N = 17;

Trie<ll, P, ALPHABET, LOG_N> trie;
typedef Trie<ll, P, ALPHABET, LOG_N>::Node HNode;

void dfs(HNode* v, const vector<ll>& hashT) {
    HNode* suff = v->calcSuffix(hashT);
    suff->addVertex(v);

    for (int i = 0; i < ALPHABET; ++i)
        if (v->to[i] != NULL)
            dfs(v->to[i], hashT);
}

void eat(const string& t, vector<int>& ans) {
    const HNode* v = trie.root;
    for (int i = 0; i < t.size() && v != NULL; ++i) {
        for (auto cand: v->relaxes) {
            int lev = cand->lev;
            if (lev < ans.size())
                ans[lev] = max(ans[lev], cand->leafs);
        }
        v = v->to[t[i] - 'a'];

    }
}

char buffer[MAX_N];
string readString() {
    scanf("%s", buffer);
    return string(buffer);
}

vector<ll> calcReverseHash(const string& t) {
    vector<ll> ret(t.size());
    ll hash = 0, pw = 1;
    int len = t.size();
    for (int i = 0; i < t.size(); ++i) {
        hash = hash + pw * (t[len - i - 1] - 'a' + 1);
        ret[i] = hash;
        pw = pw * P;
    }
    return ret;
}

int main() {
    freopen(TASK".in", "r", stdin);
    freopen(TASK".out", "w", stdout);
    int n, k;
    string t = readString();
    cin >> n >> k;
    for (int i = 0; i < n; ++i)
        trie.insert(readString());
    vector<ll> reverseHash = calcReverseHash(t);
    dfs(trie.root, reverseHash);
    int len = t.size();
    vector<int> ans(len + 1, 0);

    eat(t, ans);
    for (int i = len - 1; i >= len - k; --i) {
        if (i != len - k)
            printf("%d ", ans[i]);
        else
            printf("%d\n", ans[i]);
    }
    return 0;
}
