#include <iostream>
#include <vector>
#include <cstring>
#include <cstdio>
using namespace std;

typedef long long ll;
#define TASK "parades"

template <class HashType, int ALPHABET, int LOG_N>
struct Trie {
#define SZ ((1<<LOG_N) + 5)

    struct Node {
        Trie * trie;
        Node *parent;
        int lev, leafs, pc;
        int suffLeafs[(1<<LOG_N) + 5];

        Node* to[ALPHABET];
        Node() {}
        Node(Trie * trie): trie(trie), lev(0), leafs(0) {
            memset(to, 0, sizeof to);
            memset(suffLeafs, 0, sizeof suffLeafs);
        }

        Node(Trie * trie, Node* parent, int c): trie(trie), parent(parent), leafs(0) {
            lev = parent->lev + 1;
            pc = c;
            memset(to, 0, sizeof to);
            memset(suffLeafs, 0, sizeof suffLeafs);
        }
    };


    Node *root;
    Trie(): ptr(0) {
        root = newNode();
    }

    Node* insert(const string& s) {
        Node *v = root;
        int len = s.size();
        for (int i = 0; i < len; ++i) {
            int c = s[i] - 'a';
            if (v->to[c] == NULL)
                v->to[c] = newNode(v, c);
            v = v->to[c];
            v->leafs++;
        }
        return v;
    }
private:
    Node* newNode(Node* v, int c) {
        mem[ptr++] = Node(this, v, c);
        return &mem[ptr - 1];
    }

    Node* newNode() {
        mem[ptr++] = Node(this);
        return &mem[ptr - 1];
    }

    Node mem[SZ];
    int ptr;
};

const int MAX_N = 1e5 + 10;
const int ALPHABET = 26;
const int LOG_N = 11;

Trie<ll, ALPHABET, LOG_N> trie;
typedef Trie<ll, ALPHABET, LOG_N>::Node HNode;

char buffer[MAX_N];
string readString() {
    scanf("%s", buffer);
    return string(buffer);
}

void calcSuffs(HNode* v, const string& t) {
    HNode* cur = v;
    for (int i = t.size(); cur != NULL && i >= 0;
            --i, cur = (i >= 0 && t[i] - 'a' == cur->pc ? cur->parent : NULL))
        cur->suffLeafs[i] = v->leafs;
    for (int i = 0; i < ALPHABET; ++i)
        if (v->to[i] != NULL)
            calcSuffs(v->to[i], t);
}

int main() {
    freopen(TASK".in", "r", stdin);
    freopen(TASK".out", "w", stdout);

    int n, k;
    string t = readString();
    cin >> n >> k;
    for (int i = 0; i < n; ++i)
        trie.insert(readString());
    calcSuffs(trie.root, t);
    HNode* v = trie.root;
    vector<int> ans(k + 1, 0);
    for (int i = 0; i < t.size() && v != NULL; ++i) {
        int l = t.size() - i;
        for (int j = 1; j <= min(k, l); ++j)
            ans[j] = max(ans[j], v->suffLeafs[i + j]);
        v = v->to[t[i] - 'a'];
    }
    for (int i = 1; i <= k; ++i)
        printf("%d ", ans[i]);
    return 0;
}
