import java.io.*;
import java.util.*;

class Pair<F, S> {
    public F first;
    public S second;
    public Pair(F f, S s) {
        this.first = f;
        this.second = s;
    }
}

class Dsu {
    private int[] p;
    private int[] rank;

    public Dsu(int n) {
        p = new int[n];
        rank = new int[n];
        for (int i = 0; i < n; ++i) p[i] = i;
    }

    public int root(int v) {
        if (v == p[v])
            return v;
        p[v] = root(p[v]);
        return p[v];
    }

    public boolean join(int x, int y) {
        x = root(x);
        y = root(y);
        if (x == y)
            return false;
        if (rank[x] < rank[y]) {
            int tmp = x;
            x = y;
            y = tmp;
        }
        p[y] = x;
        if (rank[x] == rank[y]) ++rank[x];
        return true;
    }
}

class Test {
    public String t;
    int k;
    String[] a;
    public String descr;


    public Test(String t, int k, String[] a, String descr) {
        this.t = t;
        this.k = k;
        this.a = a;
        this.descr = descr;
    }

    public Test(String t, int k, List<String> a, String descr) {
        this.t = t;
        this.k = k;
        this.a = new String[a.size()];
        a.toArray(this.a);
        this.descr = descr;
    }
}

public class TestGen {

    private static final int GROUP1_N = 300;
    private static final int GROUP2_N = 2000;
    private static final int MAX_N = 100000;
    private static final int ALPABET = 26;

    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0xF00C);

    private String folder;
    private String subtask;
    private int groupNo;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void setGroupNo(int groupNo) {
        this.groupNo = groupNo;
    }

    void print(Test test) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        System.out.println("Generating #" + testCount + ": " + test.descr);

        out.println(test.t);
        out.println(String.format("%d %d", test.a.length, test.k));
        for (String s : test.a)
            out.println(s);
        out.close();

        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, ""));
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");
        isSamples = true;

        System.out.println("==== Generating samples ===");
        print(new Test("abcdef", 3, new String[]{"abcefx", "abcdfab", "abcdf"}, "sample1"));
        print(new Test("abcab", 4, new String[]{"aaba", "aabz", "ad"}, "sample2"));
        testCount = 0;
        isSamples = false;
    }

    void genHandleTests() throws IOException {
    }

    private abstract class InfIterator implements Iterator<Integer> {
        @Override
        public boolean hasNext() {
            return true;
        }
    }

    private class ConstIterator extends InfIterator {
        private int x;
        public ConstIterator(int x) {
            this.x = x;
        }

        @Override
        public Integer next() {
            return x;
        }
    }

    private class SeqIterator extends InfIterator {
        private int ptr = 0;

        @Override
        public Integer next() {
            return ++ptr;
        }
    }

    public void genSubtask(int maxT, int sumSi, int groupNo, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");
        setGroupNo(groupNo);
        System.out.println("==== Generating group #" + groupNo + " ===");

        int sq = (int)Math.sqrt(maxT);

        if (groupNo == 3) {
            print(genCyclicTueMorsa(maxT / 3, maxT - 1, sumSi));
            print(genCyclicTueMorsa(maxT / 5, maxT - 1, sumSi));
            print(genCyclicTueMorsa(maxT / 4, maxT - 1, sumSi));
        }

        print(genCyclic(maxT, maxT - 1, sumSi, 1));
        print(genCyclic(maxT / 10, maxT - 1, sumSi, 1));
        print(genCyclic(maxT / 100, maxT - 1, sumSi, 1));


        print(genCyclic(maxT / 2, maxT - 1, sumSi, 2));
        print(genCyclic(20, maxT - 1, sumSi, 3));
        print(genCyclic(maxT / 3, maxT - 1, sumSi, 11));
        print(genCyclic(maxT / 3, maxT - 1, sumSi, sq));



        if (groupNo == 2) {
            print(genCyclic(maxT / 9, maxT - 1, sumSi, 100));
            print(genCyclic(maxT / 1000, maxT - 1, sumSi, 1));
        }

        if (groupNo == 3) {
            print(genCyclic(maxT / 10, maxT - 1, sumSi, 100));
            print(genCyclic(maxT / 100, maxT - 1, sumSi, 128));
            print(genCyclic(maxT / 1212, maxT - 1, sumSi, 8));
            print(genCyclic(maxT / 10000, maxT - 1, sumSi, 1));
            print(genCyclic(maxT / 5000, maxT - 1, sumSi, 1));
        }

        print(genReal(maxT, 2, 1, maxT - 1, sumSi, 0.5, 3));
        print(genReal(maxT, 3, 1, maxT - 1, sumSi, 0.5, 5));
        print(genReal(maxT, 5, 1, maxT - 1, sumSi, 0.5, 10));

        print(genReal(maxT, 5, maxT / 2, maxT - 1, sumSi, 0.5, 5));
        print(genReal(maxT, 5, maxT / 3, maxT - 1, sumSi, 0.5, 3));
        print(genReal(maxT, 5, 2 * maxT / 3, maxT - 1, sumSi, 0.5, 6));

        print(genReal(maxT / 10, 2, 1, maxT / 10 - 1, sumSi, 0.5, 3));
        print(genReal(maxT / 100, 20, 1, maxT / 100 - 1, sumSi, 0.5, 10));
        print(genReal(maxT / 100, 100, 1, maxT / 100 - 1, sumSi, 0.5, 10));

        print(genReal(maxT / 10, 10, 1, maxT / 10 - 1, sumSi, 0.5, 10));
        print(genReal(maxT / 10, 20, 1, maxT / 10 - 1, sumSi, 0.5, 10));

        print(genReal(maxT / 10, 10, maxT / 20, maxT / 10 - 1, sumSi, 0.5, 5));
        print(genReal(maxT / 10, 100, maxT / 20, maxT / 10 - 1, sumSi, 0.5, 10));

        if (groupNo == 3) {
            print(genReal(maxT / 1000, 10, maxT / 2000, maxT / 1000 - 1, sumSi, 0.5, 5));
            print(genReal(maxT / 1000, 10, 1, maxT / 1000 - 1, sumSi, 0.5, 11));
            print(genReal(maxT / 1000, 20, maxT / 2000, maxT / 1000 - 1, sumSi, 0.5, 16));
            print(genReal(maxT / 1000, 100, maxT / 2000, 2 * maxT / 1000 / 3 - 1, sumSi, 0.5, 20));
            print(genReal(maxT / 1000, 100, maxT / 2000, 2 * maxT / 1000 / 3 - 1, sumSi, 0.5, 22));
        }

        if (groupNo >= 2) {
            int sqt = (int)Math.sqrt(maxT);
            print(genStringsLongLikeMyDick(maxT, sumSi, new ConstIterator(maxT), 1));
            print(genStringsLongLikeMyDick(maxT, sumSi, new ConstIterator(maxT), 2));

            print(genStringsLongLikeMyDick(maxT, sumSi, new ConstIterator(maxT / 2), 1));
            print(genStringsLongLikeMyDick(maxT, sumSi, new ConstIterator(maxT / 5), 2));
            print(genStringsLongLikeMyDick(maxT, sumSi, new ConstIterator(maxT / 10), 1));

            print(genStringsLongLikeMyDick(maxT, sumSi, new ConstIterator(maxT / sqt), 1));
            print(genStringsLongLikeMyDick(maxT, sumSi, new ConstIterator(maxT / sqt), 2));

            print(genStringsLongLikeMyDick(maxT, sumSi, new SeqIterator(), 1));
            print(genStringsLongLikeMyDick(maxT, sumSi, new SeqIterator(), 3));
        }

        testsDesc.println(String.format("comment_subtask%d=\"|t| &lt;= %d, sumSi &lt;= %d\"", subtaskCount, maxT, sumSi));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private Test genCyclicTueMorsa(int maxT, int k, int sum) {
        StringBuilder cycle = new StringBuilder();
        cycle.append('a');
        for (int i = 0; i < 12; ++i) {
            int sz = cycle.length();
            for (int j = 0; j < sz; ++j)
                if (cycle.charAt(j) == 'a')
                    cycle.append('b');
                else
                    cycle.append('a');
        }
        cycle = cycle.reverse();

        int cycleLen = cycle.length();
        int maxChunksT = maxT / cycleLen;
        if (maxChunksT == 0)
            throw new AssertionError("maxChunksT == 0");
        StringBuilder tBuilder = new StringBuilder();
        for (int i = 1; i < maxChunksT; ++i)
            tBuilder.append(cycle);
        tBuilder.append(cycle.substring(0, randInt(cycleLen)));
        k = Math.min(k, tBuilder.length() - 1);

        int types = randInt(2 * (int)Math.sqrt(sum / cycleLen));
        int[] part = genPartition(sum / cycleLen, types);
        //System.out.println("Partition: " + sum / cycleLen + " " + types);

        List<String> a = new ArrayList<>();
        for (int aPart : part) {
            int cnt = randDivisor(aPart);
            int ci = aPart / cnt;
            StringBuilder tmp = new StringBuilder();
            for (int j = 1; j <= ci; ++j)
                tmp.append(cycle);
            int len = cycleLen * ci;
            int l = random.nextInt(len);
            int ln = random.nextInt(Math.min(cycleLen, len - l));
            String cur = tmp.substring(0, l) + tmp.substring(l + ln);
            for (int j = 1; j <= cnt; ++j)
                a.add(cur);
        }
        shuffle(a);
        return new Test(tBuilder.toString(), k, a, String.format("genCyclicTueMorsa: maxT = %d, k = %d, sum = %d, cycleLen = %d, types = %d", maxT, k, sum, cycleLen, types));

    }

    private Test genReal(int maxT, int n, int minK, int maxK, int sum, double probTake, int kgen) {
        final int MAX_JOINS = 1_000_000_000;
        int[] lens = genPartition(sum, n);
        Dsu dsu = new Dsu(maxT + sum);
        int[] prefSum = new int[n];
        prefSum[0] = maxT;
        for (int i = 1; i < n; ++i)
            prefSum[i] = prefSum[i - 1] + lens[i - 1];

        int sumJoins = 0;
        maxK = Math.min(maxK, maxT - 1);
        int cntFound = 0;

        for (int i = 0; sumJoins < MAX_JOINS && i < kgen; ++i) {
            int j = random.nextInt(maxK - minK + 1) + minK;
            int lef = random.nextInt(maxT - j + 1);

            boolean found = false;
            for (int e = 0; e < n && sumJoins < MAX_JOINS; ++e)
                if (lens[e] >= maxT - j && random.nextDouble() < probTake) {
                    found = true;

                    for (int p = 0; p < lef; ++p, ++sumJoins)
                        dsu.join(p, prefSum[e] + p);
                    int ptr = lef;
                    for (int p = lef + j; p < maxT; ++p, ++sumJoins)
                        dsu.join(p, prefSum[e] + ptr++);
                }
            if (!found)
                --i;
            else
                ++cntFound;
        }

        char[] letters = new char[maxT + sum];
        for (int i = 0; i < maxT + sum; ++i)
            letters[i] = randomChar();
        StringBuilder tBuilder = new StringBuilder();
        for (int i = 0; i < maxT; ++i)
            tBuilder.append(letters[dsu.root(i)]);
        List<String> a = new ArrayList<>();
        for (int i = 0; i < n; i++) {
            StringBuilder sb = new StringBuilder();
            for (int j = 0; j < lens[i]; ++j)
                sb.append(letters[dsu.root(prefSum[i] + j)]);
            a.add(sb.toString());
        }
        return new Test(tBuilder.toString(), maxT - 1, a,
                String.format("genReal: maxT = %d, n = %d, minK = %d, maxK = %d, sum = %d, probTake = %.2f, sumJoins = %d, found = %d from %d",
                        maxT, n, minK, maxK, sum, probTake, sumJoins, cntFound, kgen));
    }

    private void shuffle(List<?> t) {
        Collections.shuffle(t, random);
    }

    private int[] genCombine(int n, int k) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    public int[] genPartition(int n, int k) {
        int[] comb = genCombine(n - 1, k - 1);
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    private Test genStringsLongLikeMyDick(int maxT, int sum, Iterator<Integer> it, int cycle) {
        StringBuilder t = new StringBuilder();
        for (int i = 0; i < maxT; ++i)
            t.append((char)(i % cycle + 'a'));
        List<String> a = new ArrayList<>();
        while (it.hasNext()) {
            int nx = it.next();
            if (sum < nx) break;
            sum -= nx;
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < nx; ++i)
                sb.append((char)(i % cycle + 'a'));
            a.add(sb.toString());
        }
        return new Test(t.toString(), maxT - 1, a, String.format("long strings: maxT = %d, n = %d", t.length(), a.size()));
    }

    private Test genCyclic(int maxT, int k, int sum, int cycleLen) {
        int maxChunksT = maxT / cycleLen;
        if (maxChunksT == 0)
            throw new AssertionError("maxChunksT == 0");
        String cycle = randomString(cycleLen);
        StringBuilder tBuilder = new StringBuilder();
        for (int i = 1; i < maxChunksT; ++i)
            tBuilder.append(cycle);
        tBuilder.append(cycle.substring(0, randInt(cycleLen)));
        k = Math.min(k, tBuilder.length() - 1);

        int types = randInt(2 * (int)Math.sqrt(sum / cycleLen));
        int[] part = genPartition(sum / cycleLen, types);

        List<String> a = new ArrayList<>();
        for (int aPart : part) {
            int cnt = randDivisor(aPart);
            int ci = aPart / cnt;
            StringBuilder tmp = new StringBuilder();
            for (int j = 1; j <= ci; ++j)
                tmp.append(cycle);
            int len = cycleLen * ci;
            int l = random.nextInt(len);
            int ln = random.nextInt(Math.min(cycleLen, len - l));
            String cur = tmp.substring(0, l) + tmp.substring(l + ln);
            for (int j = 1; j <= cnt; ++j)
                a.add(cur);
        }
        shuffle(a);
        return new Test(tBuilder.toString(), k, a, String.format("genCyclic: maxT = %d, k = %d, sum = %d, cycleLen = %d, types = %d", maxT, k, sum, cycleLen, types));
    }

    private int randDivisor(int x) {
        List<Integer> divs = new ArrayList<>();
        int d = 1;
        for (; d * d < x; ++d)
            if (x % d == 0) {
                divs.add(d);
                divs.add(x / d);
            }
        if (d * d == x) divs.add(d);
        return divs.get(random.nextInt(divs.size()));
    }

    private String randomString(int len) {
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < len; ++i)
            builder.append(randomChar());
        return builder.toString();
    }

    private char randomChar() {
        return (char)(random.nextInt(ALPABET) + 'a');
    }

    private int randInt(int n) {
        return random.nextInt(n) + 1;
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genSamples();
        genSubtask(GROUP1_N, GROUP1_N, 1, 21);
        genSubtask(GROUP2_N, GROUP2_N, 2, 28);
        genSubtask(MAX_N, MAX_N, 3, 51);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}