/**
 * Created by Grigorii Shovkoplias on 20.02.2016.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        new File("../preliminary").mkdir();
        desc = new PrintWriter("../tests.desc");


        done = 0;
        genSamples();

        //-------------------------first group-------------------------------------------------
        subtask = "subtask1";

        genWithoutDeletions(3, 10);
        genWithoutDeletions(4, 13);
        genWithoutDeletionsSpecial(6, 11);

        for (int i = 0; i < 5; i++) {
            genWithoutDeletions(rand(MAXN1) + 1, rand(MAXM1) + 1);
        }

        genWithoutDeletionsSpecial(MAXN1, MAXM1);
        genWithoutDeletions(MAXN1, MAXM1);
        genWithoutDeletions(MAXN1, MAXM1 - 4);

        desc.println(String.format("comment_subtask%d=\"n &lt;= %d and m &lt;= %d and no deletions\"", 1, MAXN1, MAXM1));
        desc.println(String.format("bonus_subtask%d=%d", 1, 23));

        //-------------------------second group-------------------------------------------------
        subtask = "subtask2";

        genRand(3, 10);
        genRand(4, 13);
        genRandSpecial(6, 11);

        for (int i = 0; i < 3; i++) {
            genRand(rand(MAXN2) + 1, rand(MAXM2) + 1);
            genRandSpecial(rand(MAXN2) + 1, rand(MAXM2) + 1);
            genMaxDelete(rand(MAXN2) + 1, rand(MAXM2) + 1);
            genMaxDeleteSpecial(rand(MAXN2) + 1, rand(MAXM2) + 1);
        }

        for (int i = 0; i < 2; i++) {
            genWithoutDeletionsSpecial(MAXN2, MAXM2);
            genWithoutDeletions(MAXN2, MAXM2 - 4);
            genRand(MAXN2, MAXM2);
            genRandSpecial(MAXN2, MAXM2);
            genMaxDelete(MAXN2, MAXM2);
            genMaxDeleteSpecial(MAXN2, MAXM2);
        }

        desc.println(String.format("comment_subtask%d=\"n &lt;= %d and m &lt;= %d\"", 2, MAXN2, MAXM2));
        desc.println(String.format("bonus_subtask%d=%d", 2, 24));

        //-------------------------third group-------------------------------------------------
        subtask = "subtask3";

        genRand(239, 239);
        genRand(3, 1);
        genRandSpecial(69, 119);

        for (int i = 0; i < 3; i++) {
            genRand(rand(MAXN3) + 1, rand(MAXM3) + 1);
            genRandSpecial(rand(MAXN3) + 1, rand(MAXM3) + 1);
            genMaxDelete(rand(MAXN3) + 1, rand(MAXM3) + 1);
            genMaxDeleteSpecial(rand(MAXN3) + 1, rand(MAXM3) + 1);
        }

        for (int i = 0; i < 3; i++) {
            genWithoutDeletionsSpecial(MAXN3, MAXM3);
            genWithoutDeletions(MAXN3, MAXM3 - 4);
            genRand(MAXN3, MAXM3);
            genRandSpecial(MAXN3, MAXM3);
            genMaxDelete(MAXN3, MAXM3);
            genMaxDeleteSpecial(MAXN3, MAXM3);
        }
        desc.println(String.format("comment_subtask%d=\"n &lt;= %d and m &lt;= %d\"", 3, MAXN3, MAXM3));
        desc.println(String.format("bonus_subtask%d=%d", 3, 53));


        desc.close();

    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private PrintWriter desc;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN1 = 100;
    private final int MAXM1 = 100;
    private final int MAXN2 = 5000;
    private final int MAXM2 = 10000;
    private final int MAXN3 = 100_000;
    private final int MAXM3 = 200_000;


    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private boolean coin() {
        return rnd.nextBoolean();
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }

    private class Request {
        int type, from, to, del;

        Request(int del) {
            this.type = 1;
            this.del = del;
        }

        Request(int from, int to) {
            this.type = 2;
            this.from = from;
            this.to = to;
        }

        public void print(PrintWriter out) {
            if (type == 1) {
                out.println("1 " + del);
            } else {
                out.println("2 " + from + " " + to);
            }
        }
    }

    private void print(int n, Request[] requests) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(n + " " + requests.length);
        for (Request request : requests) {
            request.print(out);
        }
        out.close();
    }


    private String getPath() {
        return String.format("../tests/%02d", done);
    }

    private void printDesc(String path, String folder, String subtask, int points, String comment) throws IOException {
        desc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
    }

    private String subtask;

    private void printDescT(String comment) throws IOException {
        printDesc(getPath(), "tests", subtask, 0, comment);
    }

    //begin generators

    private void genSamples() throws IOException {
        PrintWriter out;
        String path;
        path = "../preliminary/01";
        out = new PrintWriter(path);
        out.println("4 4");
        out.println("2 1 4");
        out.println("2 4 1");
        out.println("2 2 4");
        out.println("2 3 3");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 1");
        path = "../preliminary/02";
        out = new PrintWriter(path);
        out.println("6 5");
        out.println("1 1");
        out.println("1 2");
        out.println("1 3");
        out.println("2 1 3");
        out.println("2 1 1");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 2");
    }


    private void shuffle(int[] a) {
        for (int i = 0; i < a.length; i++) {
            int j = rand(a.length);
            int tmp = a[j];
            a[j] = a[i];
            a[i] = tmp;
        }
    }

    //begin generators

    private void genRand(int n, int m) throws IOException {
        Request[] requests = new Request[m];
        int[] stairs = new int[n];
        for (int i = 0; i < n; i++)
            stairs[i] = i;
        shuffle(stairs);
        int cnt = 0;
        for (int i = 0; i < m; i++) {
            if (coin() && cnt < n - 1) {
                requests[i] = new Request(stairs[cnt++] + 1);
            } else {
                requests[i] = new Request(rand(n) + 1, rand(n) + 1);
            }
        }
        print(n, requests);
        printDescT(String.format("Random: n = %d, m = %d", n, m));
    }

    private void genWithoutDeletions(int n, int m) throws IOException {
        Request[] requests = new Request[m];
        for (int i = 0; i < m; i++) {
            requests[i] = new Request(rand(n) + 1, rand(n) + 1);

        }
        print(n, requests);
        printDescT(String.format("Random without deletions: n = %d, m = %d", n, m));
    }

    private void genWithoutDeletionsSpecial(int n, int m) throws IOException {
        Request[] requests = new Request[m];
        for (int i = 0; i < m; i++) {
            int k = rand(n) + 1;
            requests[i] = new Request(k, k);

        }
        print(n, requests);
        printDescT(String.format("Equals without deletions: n = %d, m = %d", n, m));
    }

    private void genMaxDelete(int n, int m) throws IOException {
        Request[] requests = new Request[m];
        int[] stairs = new int[n];
        for (int i = 0; i < n; i++)
            stairs[i] = i;
        shuffle(stairs);
        int cnt = 0;
        for (int i = 0; i < m; i++) {
            if (cnt < n - 1) {
                requests[i] = new Request(stairs[cnt++] + 1);
            } else {
                requests[i] = new Request(rand(n) + 1, rand(n) + 1);
            }
        }
        print(n, requests);
        printDescT(String.format("Random max deletions: n = %d, m = %d", n, m));
    }


    private void genRandSpecial(int n, int m) throws IOException {
        Request[] requests = new Request[m];
        int[] stairs = new int[n];
        for (int i = 0; i < n; i++)
            stairs[i] = i;
        shuffle(stairs);
        int cnt = 0;
        for (int i = 0; i < m; i++) {
            if (coin() && cnt < n - 1) {
                requests[i] = new Request(stairs[cnt++] + 1);
            } else {
                int k = rand(n) + 1;
                requests[i] = new Request(k, k);
            }
        }
        print(n, requests);
        printDescT(String.format("Random all equals: n = %d, m = %d", n, m));
    }

    private void genMaxDeleteSpecial(int n, int m) throws IOException {
        Request[] requests = new Request[m];
        int[] stairs = new int[n];
        for (int i = 0; i < n; i++)
            stairs[i] = i;
        shuffle(stairs);
        int cnt = 0;
        for (int i = 0; i < m; i++) {
            if (cnt < n - 1) {
                requests[i] = new Request(stairs[cnt++] + 1);
            } else {
                int k = rand(n) + 1;
                requests[i] = new Request(k, k);
            }
        }
        print(n, requests);
        printDescT(String.format("Random all equals and max deletions: n = %d, m = %d", n, m));
    }


    //end generators

}
